export class Defer<T = any> {
    resolve: (data?: T) => void;
    reject: (error: Error | string) => void;
    promise: Promise<T>;
}
/**
 * @deprecated use `new Defer()` directly, since v1.0.0.
 */
export const makeDefer: <T>() => Defer<T>;
export type EventHandler = (data: any) => void;
export interface EventTrigger {
    addEventListener?(name: string, handler: EventHandler): void;
    removeEventListener?(name: string, handler: EventHandler): void;
    on?(name: string, handler: EventHandler): this;
    off?(name: string, handler: EventHandler): this;
}
declare global {
    interface SymbolConstructor {
        readonly observable: unique symbol;
    }
}
export interface Observer<T = any> {
    next(value: T): void;
    error(reason: string | Error): void;
    complete(): void;
}
export interface Subscription {
    unsubscribe(): void;
    readonly closed: boolean;
}
export interface Subscribable<T = any> {
    [Symbol.observable](): Subscribable<T>;
    subscribe(onNext: Observer<T>['next'], onError?: Observer<T>['error'], onComplete?: Observer<T>['complete']): Subscription;
}
export type SubscriberFunction<T = any> = (observer: Observer<T>) => (() => void) | void;
export class Observable<T = any> implements Subscribable {
    constructor(subscriber: SubscriberFunction<T>);
    [Symbol.observable](): this;
    [Symbol.asyncIterator](): AsyncGenerator<Awaited<T>, void, unknown>;
    static fromStream<T>(list: Iterable<T> | AsyncIterable<T>): Observable<T>;
    static of<T = any>(...items: T[]): Observable<T>;
    toPromise(): Promise<any>;
    subscribe(onNext: Observer<T>['next'], onError?: Observer<T>['error'], onComplete?: Observer<T>['complete']): {
        unsubscribe: () => boolean;
        readonly closed: boolean;
    };
    static from<T = any>(observable: Iterable<T> | AsyncIterable<T> | Subscribable<T>): Observable<T>;
    static fromEvent<T = any>(target: EventTrigger, name: string): Observable<T>;
}
export function createQueue<D = any>(): {
    process<R = any>(data: D): Promise<R>;
    destroy(): void;
    observable: Observable<{
        defer: Defer;
        data: D;
    }>;
};

//# sourceMappingURL=index.d.ts.map
